///////////////////////////////////////////////////////////////
//                                                           //
//  CFileInfo.cpp  Store necessory information about a file  //
//				   inlcuding which files has been used in    //
//				   the file                                  //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//  Application:display all file duplicates in a specified	 //
//			   directory and all its sub directories. Also 	 //
//			   provides a functionality to display all files //
//			   under a specified directory and all its		 //
//			   subdirectories.								 //	
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                    copyright (c) 1999                     //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma warning( disable : 4786 )
#include <fstream>
#include "CFileInfo.h"



//Contructor of the class CFileInfo
CFileInfo::CFileInfo(const char * _filename){
	//copy the _filename to strFileName		
	strFileName = new char[strlen(_filename)+1];
	if(!strFileName)
		throw "\n memory allocation failure\n";
	strcpy(strFileName,_filename);
}

//Decontructor of the class CFileInfo
CFileInfo::~CFileInfo(){
	delete [] strFileName;

}

//return the list of included files
ROW CFileInfo::getList(){
	return rIncludeList;
}


//add files ,which strFileName includes, to a list
void CFileInfo::makeIncludeList(){
	static ifstream in;
	rIncludeList.push_back(strFileName);
	in.open(strFileName);
	string line;
	while(in.good()){
		getline(in,line);
		int pos;
		if(foundHash(line)&& foundDelim(line,pos)){
			string name = extractName(line,pos);
			rIncludeList.push_back(name);
		}		
	}
	in.close();
	in.clear();

}

//check whether string s contains '#',if so, return true, otherwise, return false;
bool CFileInfo::foundHash(const string &s){
	return (s[0] == '#');
}

//check whether string s contains '<' or '\',if so, return true, otherwise, return false;
bool CFileInfo::foundDelim(const string &s ,int &pos){
	pos = s.find_first_of('<');
	if(pos == string::npos){
		pos = s.find_first_of('\"');
	}
	return (pos != string::npos);
			
}

string CFileInfo::extractName(const string &s,int pos){
	string temp;
	while(pos<s.size()){
		temp += s[pos++];
		if(s[pos] == '>' || s[pos] == '\"'){
			temp +=s[pos];
			break;
		}
	}	
	return temp;
}


#ifdef TEST_CFILEINFO
#pragma warning( disable : 4786 )

//----< test stub >--------------------------------------------

void main( ) {
	cout << "this is the test stub in class CFileInfo"<<"\n";
	CFileInfo c("Basic.h");
	c.makeIncludeList();
	ROW l = c.getList();
	for(int i=0;i<l.size();i++)
		cout<<l[i];
    
}

#endif

