///////////////////////////////////////////////////////////////
//                                                           //
//  CFilePattern.cpp  Analyze file pattern , then find out   //
//				      related files;                         //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//  Application:display all file duplicates in a specified	 //
//			   directory and all its sub directories. Also 	 //
//			   provides a functionality to display all files //
//			   under a specified directory and all its		 //
//			   subdirectories.								 //	
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                    copyright (c) 1999                     //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////

#include <string>
#include <iostream>
#include <windows.h>
#include "CFilePattern.h"



//constructor of CFilePattern
CFilePattern::CFilePattern(const char *filemask){	
	pFileMask = new char[strlen(filemask)+1];
	if(!pFileMask)
		throw "\n memory allocation failure \n";
	strcpy(pFileMask,filemask);	
}

//decontructor of CFilePattern
CFilePattern::~CFilePattern(){
	delete [] pFileMask;
}

//return the file list related to this file pattern
ROW CFilePattern::getList(){
	return rFileNames;
}

//find out the files related to the file pattern, and insert them to a list
//if can not find anything, just return false, otherwise return true
bool CFilePattern::makeFileNameList(){	
	char curDir[256];
	if(!GetCurrentDirectory(256,curDir)){
		cerr<<"Couldn't get the current directory.";
		return false;
	}
	addFileToList(curDir);
	return true;
}

//add file to the file list
void CFilePattern::addFileToList(char * dirName){
	char *fileName;
	char curDir[ 256 ];
	char fullName[ 256 ];
	HANDLE fileHandle;
	WIN32_FIND_DATA findData;
	if( !GetCurrentDirectory( 256, curDir) ) 
		return;		
	if( strcmp( dirName, "." ) && strcmp( dirName, ".." ) )
	{
		if( !SetCurrentDirectory( dirName ) ) 
			return;
	}
	else	return;
	if( !GetFullPathName( pFileMask, 256, fullName,&fileName ) ) 
		return;
	// Loop through all files in the directory
	fileHandle = FindFirstFile( pFileMask,	&findData );
	while ( fileHandle != INVALID_HANDLE_VALUE )
	{
		// If the name is a directory,recursively walk it. Otherwise print the file's data
		if( findData.dwFileAttributes &	FILE_ATTRIBUTE_DIRECTORY )		
			addFileToList( findData.cFileName);		
		else{
			string tempstring(findData.cFileName);
			rFileNames.push_back(tempstring);		
		}
		if (!FindNextFile( fileHandle, &findData ))
			break;
 	}	
	FindClose( fileHandle );
	SetCurrentDirectory( curDir );
}

#ifdef TEST_CFILEPATTERN
#pragma warning( disable : 4786 )

//----< test stub >--------------------------------------------

void main( ) {
	cout << "this is the test methods in class CFilePattern"<<"\n";
	ROW templist;
	CFilePattern c(*.*);    
	c.MakeFileNameList();
	templist = c.getList();
	for(int i=0;i<templist.size();i++)
		cout<<templist[i];
}

#endif

