///////////////////////////////////////////////////////////////
//                                                           //
//  CGrid.cpp  define and implementation of class CGrid      //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//  Application:display all file duplicates in a specified	 //
//			   directory and all its sub directories. Also 	 //
//			   provides a functionality to display all files //
//			   under a specified directory and all its		 //
//			   subdirectories.								 //	
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                    copyright (c) 1999                     //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma warning(disable : 4786)
#include "Basic.h"
#include "CGrid.h"

//Constructor of class CGird
CGrid::CGrid(){
}

//Deconstructor of class CGird
CGrid::~CGrid(){
}

//Swap two rows in class CGird
void CGrid::Swap(int row1,int row2){	
	if(0<=row1 && row1 <gGrid.size())
		if(0<=row1 && row1 <gGrid.size()){
			ROW temp = gGrid[row1];
			gGrid[row1] = gGrid[row2];
			gGrid[row2] = temp;
		}	
}

//add a row to Grid;
void CGrid::Push(ROW r){
	gGrid.push_back(r);
}


//replace the element at insertPos by a row numbered rowNum.
//if repeatCheck is true, I will add certain element only once.
void CGrid::ExtractElement(int i,int j,int rowNum,bool repeatCheck){
	//todo repeatCheck
	gGrid[i].erase(gGrid[i].begin()+j);
	int index;
	for(index=0;index<gGrid[rowNum].size();index++){
		if (!repeatCheck){
			gGrid[i].insert(gGrid[i].begin()+j,gGrid[rowNum][index]);
			j++;
		}else{
			bool found=false;
			for(int k=0;k<gGrid[i].size();k++){
				if (gGrid[i][k] ==gGrid[rowNum][index]){
					found = true;
				}
			}
			if (!found){
				gGrid[i].insert(gGrid[i].begin()+j,gGrid[rowNum][index]);
				j++;
			}
	
		}
	}

}

//return the number of Row which includes string s, if not found, return -1
int CGrid::GetIncludeRowNum(string& s){
	int i;
	for(i=0;i<gGrid.size();i++){
		string tempString = gGrid[i][0];
		int _size=s.size();
		string s_sub= s.substr(1,_size-2);
		if((tempString.compare(s)==0)||(tempString.compare(s_sub) == 0))
			return i;
	}	
	return -1;
}

//merge every element in Grid
void CGrid::Merge(){
	Sort(ALPHABET);
	int i;
	//for every row in this grid
	for(i=0;i<gGrid.size();i++){
		int j;
		//for every element in this row except the first one
		for(j=1;j<gGrid[i].size();j++){
			RecursiveExtract(i,j);
		}
		
	}
	Sort(DEPENDENCE);
}

//Recursively extract every element in Grid
void CGrid::RecursiveExtract(int i,int j){
	string tempString = gGrid[i][j];	
	//if tempString is NOT in command line, getIncludeRowNum will return -1;
	int rowNum = GetIncludeRowNum(tempString);
	if (rowNum !=-1){				
		int index;
		for(index=1;index<gGrid[rowNum].size();index++){
			RecursiveExtract(rowNum,index);
		}		
		ExtractElement(i,j,rowNum,true);
	}
}

//if row r1 in included by row r2, then return true.otherwise return false;
bool CGrid::Dependence(int r1,int r2){
	//every element in row r2;
	int i;
	for(i=0;i<gGrid[r2].size();i++){
		int j;
		for(j=0;j<gGrid[r1].size();j++){
			if (gGrid[r2][i].compare(gGrid[r1][0]) ==0)
				return true;
		}	
	
	}
	return false;
}

//sort the Grid , now support two types:ALPHABET and DEPENDENCE
void CGrid::Sort(SORT_TYPE sortType){
		int i;
		for(i=1;i<gGrid.size();i++){
			bool up = true;
			int index= i-1;
			int now = i;


			while(up && (index >=0)){
				bool _swap=false;

				switch(sortType){
					case ALPHABET:{
						if (gGrid[now][0].compare(gGrid[index][0]) >0)
							up = false;					
						else
							_swap = true;
						break;

					}						
					case DEPENDENCE:{
						if(Dependence(now,index))
							up = false;
						else 
							_swap = true;
						break;
					}					
				}

				if (_swap){
					Swap(now,index);
					now = index;
				}
				index--;
			}
		}	
}

//display the elements of CGrid;
void CGrid::Display(){
	int i;
	for(i=0;i<gGrid.size();i++){
		cout<<" "<<gGrid[i][0]<<"\n";
		int j;
		for(j=1;j<gGrid[i].size();j++){
			cout<<"        |----"<<gGrid[i][j]<<endl;
		}
		cout<<"\n\n";
	}
}

#ifdef TEST_CGRID
#pragma warning( disable : 4786 )

//----< test stub >--------------------------------------------
void main( ) {
	cout << "this is the test methods in class CGrid"<<"\n";
	RAW    test,row2;
	test.push_back("testtest");
	row2.push_back("row2");
	CGrid  c;
    c.Push(test);
	c.Display();
	c.Sort(ALPHABET);
	c.Display();
}

#endif


