///////////////////////////////////////////////////////////////
//                                                           //
//  Sweep.cpp    Main file of the project                   //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//                                                           //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//                                                           //
//  Application: Clean the files generated					 //
//               by Visual C++                               //
//               and display informatino about duplicate     //
//               files                                       //
//                                                           //
//                                                           //
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                    copyright (c) 1999                     //
//                                                           //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////
#pragma warning (disable : 4786)


#include "CNavigator.h"
#include "CCleanFile.h"
#include "CCleanFileCall.h"
#include "CDupFile.h"
#include <iostream>
#include <windows.h>

using namespace std;

//constructor of CCleanFile with a directory name
CCleanFile::CCleanFile(string _root){
	rootDirectory = _root;
	extList.push_back("dep");
	extList.push_back("ncb");
	extList.push_back("tli");
	extList.push_back("aps");
	extList.push_back("opt");
	extList.push_back("plg");
	extList.push_back("obj");
	extList.push_back("ilk");
	extList.push_back("pch");
	extList.push_back("pdb");
	extList.push_back("idb");
	extList.push_back("tlh");
}

//deconstructor of CCleanFile
CCleanFile::~CCleanFile(){
	extList.clear();
}

//print out the file tree 
void CCleanFile::printFileTree(){	
	cout<<"\n--------------------------------------------------\n\n---------------------------------------------------\n";
	for(int i=0;i<(*pFileTree).size();i++)
		for(int j=0;j<(*pFileTree)[i].size();j++){			
			CFileNode tempNode;
			tempNode = (CFileNode)(*pFileTree)[i][j];
			//if not directroy and isToBeDeleted, then print out
			if((!tempNode.isDirectory)&&(tempNode.isToBeDeleted)){
				string _fullName = getFullFileName(tempNode);
				cout<<"|----- "<<_fullName<<"\n";
			}			
		}
	cout<<"\n---------------------------------------------------\n\n---------------------------------------------------\n";

}


/*
Function : Return the full path of a file node. 
Input	 : fileNode
Output   : string (full path of this fileNode)
*/
string CCleanFile::getFullFileName(CFileNode& _fileNode){
	int _level,_index,_parentIndex;

	_level = _fileNode.levelNumber;
	_index = _fileNode.indexNumber;
	_parentIndex = _fileNode.parentIndexNumber;

	string temp=_fileNode.fileName;
	while(_level>0){		
		CFileNode tempNode = (CFileNode)(*pFileTree)[--_level][_parentIndex];		
		//todo , whether I should add rootdir to every file?		
		temp= tempNode.fileName+"\\"+temp;
        _parentIndex = tempNode.parentIndexNumber;
	}
	temp = rootDirectory+"\\"+temp;
	return temp;
}

/*
Function : Delete the files whose isToBeDeleted = true,and set isValidate to false in fileTree 
Input	 : fileNode
Output   : 
*/

void CCleanFile::doCleanFile( vector< vector<CFileNode> > *_pFileTree){
	pFileTree = _pFileTree;
	checkFile();
	cout<<"Here are files to be deleted ! ";
	printFileTree();
	if(userDeleteConfirm()){
		deleteFile();
	}
	return;
}


/*
Function : User Interface for user to confirm the file to be deleted 
Input	 : 
Output   : yes(true) or no(false)
*/
bool   CCleanFile::userDeleteConfirm(){
	string s ="";
	cout<<"Are you sure you want to delete these files?(Y/N)\n";
	cin>>s;
	if( s=="Y" || s=="y")
		return true;
	return false;
}


/*
Function : Delete the files whose isToBeDeleted = true,and set isValidate to false in fileTree 
Input	 : fileNode
Output   : 
*/
bool   CCleanFile::deleteFile(){
	for(int i=0;i<(*pFileTree).size();i++)	
		for(int j=0;j<((*pFileTree)[i]).size();j++){
			CFileNode tempNode = (CFileNode)((*pFileTree)[i])[j];
			//if isToBeDeleted = true,then invoke API to delete it.And set isValidate to false;	
			if (tempNode.isToBeDeleted){				
				string tempName = getFullFileName(tempNode);
				//delete file by api
				if(!DeleteFile(tempName.c_str()))
					cout<<"Can not delete file : "<<tempNode.fileName;
				else (((*pFileTree)[i])[j]).isValid = false;
			}
			
		}
		
	return true;
} 	
//check the file, if it's the type to be deleted, set isToBeDeleted = true;
void CCleanFile::checkFile(){
	for(int i=0;i<(*pFileTree).size();i++)	
		for(int j=0;j<((*pFileTree)[i]).size();j++){
			CFileNode tempNode = (CFileNode)((*pFileTree)[i])[j];
			//if isToBeDeleted = true,then invoke API to delete it.And set isValidate to false;	
			int pos = tempNode.fileName.find(".");
			if (pos<0)
				continue;
			
			if (isToBeDeleted(tempNode.fileName.substr(pos+1,3))){				
				(((*pFileTree)[i])[j]).isToBeDeleted = true;
			}
			
		}
}
//check whether we should delete file with the extension ext,if so, return true
//otherwise , return false
bool CCleanFile::isToBeDeleted(string ext){
	for(int i=0; i<extList.size(); i++){
		if (ext==extList[i])
			return true;
	}
	return false;
}

#ifdef TEST_CCLEANFILE
#pragma warning( disable : 4786 )
//----< test stub >--------------------------------------------
void main( ) {
	cout << "this is the test stub in class CCleanFile"<<"\n";
	cout<<"Test Class CNavigator"<<endl;
	cout<<"Please input a directory name"<<endl;
	string s;
	cin>>s;
	if(!strcmp(s.c_str(), ".")|| !strcmp(s.c_str(), "..")){

		if(!SetCurrentDirectory(s.c_str()))
			return;
		char buf[256];
		if(!GetCurrentDirectory(256, buf))
			return;
		s = buf;
		cout<<s<<endl;
	}	
	CNavigator n(s);
	n.createFileTree();
	n.printFileTree();
	CCleanFile c(s);
	c.doCleanFile(n.getFileTree());
}
#endif
