///////////////////////////////////////////////////////////////
//                                                           //
//  CDupFile.cpp Deal with the dupalicat files in directory  //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//                                                           //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//                                                           //
//  Application: Using to get the duplicate file info        //
//               and support to display them                 //
//                                                           //
//                                                           //
//                                                           //
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                                                           //
//                    copyright (c) 1999                     //
//                                                           //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma  warning (disable : 4786)

#include "CNavigator.h"
#include "CCleanFile.h"
#include "CCleanFileCall.h"
#include "CDupFile.h"

//constructor CDupFile
CDupFile::CDupFile(string _root, vector< CSameFileNode >*   _pSameFileList){
	rootDirectory = _root;
	pSameFileList = _pSameFileList;
}

//deconstructor CDupFile
CDupFile::~CDupFile(){
}

//public interface  of CDupFile
void CDupFile::doDupFile(vector< vector<CFileNode> >* _pFileTree){
	pFileTree = _pFileTree;
	showDupFile();
}

//display the duplicate files
void CDupFile::showDupFile(){
	for(int i=0;i<(*pSameFileList).size();i++){
		int _count = ((*pSameFileList)[i]).count;
		if( _count <=1)
			continue;
		cout<<"\n---------------------------------------------------------------------------\n";		
		cout<<" There are "<<_count<<" different files named as "<<(((*pSameFileList)[i]).fileName)<<endl;
		cout<<"\n---------------------------------------------------------------------------\n";
		for(int j=0;j<(*pSameFileList)[i].posList.size();j++){
			CFileNodePos tempPosNode =(CFileNodePos)( (((*pSameFileList)[i]).posList)[j]);
			CFileNode tempFileNode = (CFileNode)((*pFileTree)[tempPosNode.levelNumber][tempPosNode.indexNumber]);			
			cout<<"     |----- "<<getFullFileName(tempFileNode)<<endl;
		}
	}
}

//return the full path name of a file
string CDupFile::getFullFileName(CFileNode& _fileNode){
	int _level,_index,_parentIndex;

	_level = _fileNode.levelNumber;
	_index = _fileNode.indexNumber;
	_parentIndex = _fileNode.parentIndexNumber;

	string temp=_fileNode.fileName;
	while(_level>0){		
		CFileNode tempNode = (CFileNode)(*pFileTree)[--_level][_parentIndex];		
		//todo , whether I should add rootdir to every file?		
		temp= tempNode.fileName+"\\"+temp;
        _parentIndex = tempNode.parentIndexNumber;
	}
	temp = rootDirectory+"\\"+temp;
	return temp;
}

#ifdef TEST_CDUPFILE
#pragma warning( disable : 4786 )
//----< test stub >--------------------------------------------
void main( ) {
	cout<<"Test Class CNavigator"<<endl;
	cout<<"Please input a directory name"<<endl;
	string s;
	cin>>s;
	if(!strcmp(s.c_str(), ".")|| !strcmp(s.c_str(), "..")){
		if(!SetCurrentDirectory(s.c_str()))
			return;
		char buf[256];
		if(!GetCurrentDirectory(256, buf))
			return;
		s = buf;
		cout<<s<<endl;
	}	
	CNavigator n(s);
	n.createFileTree();	
	n.printFileTree();
	CDupFile d(s,n.getSameFileList());
	d.doDupFile(n.getFileTree());
	return;
}
#endif
