///////////////////////////////////////////////////////////////
//                                                           //
//  CNavigator.cpp traverse the directory and get information//
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//                                                           //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//  System:      Window NT, DOS 6.2                          //
//                                                           //
//  Application: Traverse the directory and get the informaion//
//               of file tree structure and duplicate file   //
//               information                                 //
//                                                           //
//                                                           //
//                                                           //
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                    copyright (c) 1999                     //
//                                                           //
//    all rights granted to users except right to publish    //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma warning(disable : 4786)

#include "CNavigator.h"
#include "CCleanFile.h"
#include "CCleanFileCall.h"
#include "CDupFile.h"


using namespace std;


//constructor
CNavigator::CNavigator(){
	rootDirectory = ".";	
}

//constructor
CNavigator::CNavigator(string _rootDirectory){
	rootDirectory = _rootDirectory;	
}

//deconstructor
CNavigator::~CNavigator(){
	fileTree.clear();
	sameFileList.clear();
	commandList.clear();
}

//regist a command
void CNavigator::registCommand(CCommand* _cmd){
	commandList.push_back(_cmd);
}

//invoke commands one by one
void CNavigator::invokeCommand(){
	for(int i=0; i<commandList.size(); i++)
		commandList[i]->execute(&fileTree);

}

//print out the information of files with same name
void CNavigator::printSameFileList(){
	for(int i=0;i<sameFileList.size();i++){		
		CSameFileNode tempNode;
		tempNode = (CSameFileNode)sameFileList[i];
		cout<<"\nFile"<<i<<"------"<<(tempNode.fileName).c_str()<<" #"<<tempNode.count;		
		for(int j=0;j<tempNode.posList.size();j++){
			cout<<" ("<<tempNode.posList[j].levelNumber<<","<<tempNode.posList[j].indexNumber<<" )";
		}
		cout<<endl;
	}

}

/*
Function:	traverse direcctory recursivly, then create the fileTree and sameFileNameList;
Input	:   string of root directory
Output	:
*/
void CNavigator::createFileTree(){
	
	int level = 0;
	int indexInLevel = 0;	
	vector<int> indexNo;
	char  curDir[256] ;

		if(!SetCurrentDirectory(rootDirectory.c_str()))
			return;
		if(!GetCurrentDirectory(256, curDir))
			return;
		rootDirectory=curDir;
	sameFileList.clear();
	fileTree.clear();
	traverseDirectory(curDir,"*.*",&level,indexInLevel,&indexNo);
}
//traverse directory recursively,and store info about file tree and duplicate files
void CNavigator::traverseDirectory(char *dirName,char *fileMask, int* level,int parentIndex,vector<int>* pIndexNO){
	char *fileName;
	char curDir[ 256 ];
	char fullName[ 256 ];
	HANDLE fileHandle;
	WIN32_FIND_DATA findData;
	// save current dir so it can restore it
	if( !GetCurrentDirectory( 256, curDir) ) 
		return;	
	// if the directory name is neither . or .. then change to it, otherwise ignore it
	if( strcmp( dirName, "." ) && strcmp( dirName, ".." ) )	{
		if( !SetCurrentDirectory( dirName ) ) 
			return;	
	}else 	return;

	// print out the current directory name
	if( !GetFullPathName( fileMask, 256, fullName,&fileName ) ) return;
	fileHandle = FindFirstFile( fileMask,&findData );
	int indexInLevel = 0;	
	if ((*level) < (pIndexNO->size()) )
		indexInLevel = (*pIndexNO)[*level];
	else
		pIndexNO->push_back(indexInLevel);		
	while ( fileHandle != INVALID_HANDLE_VALUE )
	{		
		if( findData.dwFileAttributes &	FILE_ATTRIBUTE_DIRECTORY )
		{
			putFindFileInfo( &findData,level ,indexInLevel,parentIndex);		
			(*level)++;
			traverseDirectory( findData.cFileName,fileMask ,level,indexInLevel,pIndexNO);
			(*level)--;
			if ( !strcmp(findData.cFileName,".")  || !strcmp(findData.cFileName,"..") )
				indexInLevel--;
		}else	putFindFileInfo( &findData,level ,indexInLevel,parentIndex);				
		indexInLevel++;
  		// loop thru remaining entries in the dir
		if (!FindNextFile( fileHandle, &findData ))			break;
 	}	
	(*pIndexNO)[*level]=indexInLevel;		
	FindClose( fileHandle );
	SetCurrentDirectory( curDir );
}

//recored informatino while traversing file system.
void CNavigator::putFindFileInfo(WIN32_FIND_DATA *findData,int* level,int index,int parentIndex){
	updateFileTree( findData,level ,index,parentIndex);		
	updateSameFileList( findData,level ,index,parentIndex);		
}

//update file tree information 
void CNavigator::updateFileTree(WIN32_FIND_DATA *findData,int* level,int index,int parentIndex){
	if (!strcmp(findData->cFileName,".")  || !strcmp(findData->cFileName,".."))
		return;	
	//deal with fileTree
	CFileNode  newFileNode,tempNode;
	newFileNode.indexNumber = index;
	newFileNode.levelNumber = *level;
	newFileNode.parentIndexNumber = parentIndex;
	newFileNode.fileName = findData->cFileName;	
	newFileNode.isToBeDeleted = false;
	newFileNode.isValid = true;
	newFileNode.isDirectory = false;
	if( findData->dwFileAttributes  & FILE_ATTRIBUTE_DIRECTORY )
		newFileNode.isDirectory = true;  
	if (newFileNode.levelNumber>=fileTree.size()){
		vector<CFileNode> newVector;
		newVector.push_back(newFileNode);
		fileTree.push_back(newVector);
	}else	(fileTree[*level]).push_back(newFileNode);
	
}

//update duplicate file information
void CNavigator::updateSameFileList(WIN32_FIND_DATA *findData,int* level,int index,int parentIndex){
	if (!strcmp(findData->cFileName,".")  || !strcmp(findData->cFileName,".."))
		return;	
	//deal with sameFileList
	for(int i=0;i<sameFileList.size();i++){
		CSameFileNode tempNode = (CSameFileNode)sameFileList[i];		
		if (!strcmp(findData->cFileName,tempNode.fileName.c_str())){			
			sameFileList[i].count++;
			CFileNodePos tempPos;
			tempPos.levelNumber = *level;
			tempPos.indexNumber = index;			
			sameFileList[i].posList.push_back(tempPos);			
			return;
		}
	}
	CFileNodePos   newPos;
	newPos.levelNumber = *level;
	newPos.indexNumber = index;
	CSameFileNode newNode;
	newNode.fileName = findData->cFileName;
	newNode.count	 = 1;
	newNode.posList.push_back(newPos);
	sameFileList.push_back(newNode);
}

//print out the stored information about file tree
void CNavigator::printFileTree(){
	cout<<"\n--------------  File Tree  ------------------------\n\n---------------------------------------------------\n";
	for(int i=0;i<fileTree.size();i++)
		for(int j=0;j<fileTree[i].size();j++){
			CFileNode tempNode;
			if (!tempNode.isValid)
				continue;
			tempNode = fileTree[i][j];
			if (tempNode.isDirectory)
				cout<<"Directory---";
			else 
				cout<<"|----------------";
			cout<<"Node   "<<tempNode.fileName<<"("<<tempNode.levelNumber<<","<<tempNode.indexNumber<<")"<<endl;
		}
	cout<<"\n---------------------------------------------------\n\n---------------------------------------------------\n";
}

//return a pointer to file tree
vector< vector<CFileNode> >* CNavigator::getFileTree(){
	return &fileTree;
}

//return a pointer to sameFileList
vector<CSameFileNode>*	CNavigator::getSameFileList(){
	return &sameFileList;
}

#ifdef TEST_CNAVIGATOR
void main(int argc, char *argv[]){
	//test stub
	cout<<"Test Class CNavigator"<<endl;
	cout<<"Please input a directory name"<<endl;
	string s;
	cin>>s;
	if(!strcmp(s.c_str(), ".")|| !strcmp(s.c_str(), "..")){
		if(!SetCurrentDirectory(s.c_str()))
			return;
		char buf[256];
		if(!GetCurrentDirectory(256, buf))
			return;
		s = buf;
		cout<<s<<endl;
	}	
	CNavigator n(s);
	n.createFileTree();	
	n.printFileTree();
	//constructor a object of CCleanFile
	CCleanFile c(s);
	c.doCleanFile(n.getFileTree());
	//constructor a object of CDupFile
	CDupFile d(s,n.getSameFileList());
	d.doDupFile(n.getFileTree());
	return;
}
#endif