///////////////////////////////////////////////////////////////
//                                                           //
//  CTutor.cpp   Tutor provide query for user from a file    //
//               by some keywords                            //
//  v1.0                                                     //
//               Manual Page                                 //
//                                                           //
//  Language:    Visual C++, ver 6.0                         //
//                                                           //
//                                                           //
//                                                           //
//  System:      Window NT, DOS 6.2                          //
//                                                           //
//                                                           //
//                                                           //
//  Application: user input some keyword, then tutor         //
//			     can return the query result                 //
//                                                           //
//  Author:    Jin,Jianxiang, Syracuse University            //
//				jjin@npac.syr.edu							 //	
//                                                           //
//                                                           //
//                    copyright (c) 1999                     //
//                                                           //
//                                                           //
//    all rights granted to users except right to publish    //
//                                                           //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma warning(disable : 4786)
#include "CTutor.h"

using namespace std;

//constructor
CTutor::CTutor(){
}
//destructor
CTutor::~CTutor(){
	resetList();
}

//print out the hit record of every block
void CTutor::printOutHitList(){
	HITMAP::iterator tempIterator;
	tempIterator = blockHitTimes.begin();
	cout<<"---------------------------------------------------\n";
	cout<<"---------------------------------------------------";	
	while(tempIterator != blockHitTimes.end()){
		cout<<"\n BlockID :"<<(*tempIterator).first<<" Hittimes:"<<(*tempIterator).second;
		tempIterator++;
	}
}
//print out the #_no block, if _titleOnly is true, only print title of this block,
//otherwise, print whole block out
void CTutor::printOutBlock(int _no,bool _titleOnly){
	if(_no<0)
		return;
	int i = _no;
	if(i>=blockList.size()){
		cout<<"Error number,try again!\n";
		return;
	}
	CBlockNode tempNode;
	tempNode = (CBlockNode)blockList[i];
	cout<<"\nBlock ID      : "<<tempNode.blockID;
	cout<<"\nBlock Title   : "<<tempNode.blockTitle;
	if (_titleOnly){
		cout<<endl;
		return;
	}
	cout<<"\nBlock Text    : "<<tempNode.blockText;
	cout<<"\nBlock Keywords\n";
	for(int j=0;j<tempNode.keywordIncluded.size();j++)
		cout<<"         |-------- "<<(tempNode.keywordIncluded)[j]<<endl;
}

//Parse a string inputed by user to server words separated by space
vector<string>  CTutor::stringToWords(string _input){
	vector<string> _v;
	_v.clear();
	int beginPos=0,endPos=0;
	while(endPos!=_input.size()){	
		beginPos = _input.find_first_not_of(" ",endPos);
		if (beginPos==-1)
			return _v;
		endPos   = _input.find_first_of(" ",beginPos);
		if (endPos==-1)	
			endPos=_input.size();
		_v.push_back(_input.substr(beginPos,(endPos-beginPos)));
	}
	return _v;

}

//read in the keywords user input
void CTutor::readInputKeywords(){
	//init keywordList
	int i=0;
	string s1="";;	
	char  temp[256];
	temp[0]='\0';
	cout<<"Please input keywords for search(example : class virtual object):\n";	
	cin.getline(temp,256,'\n');
	s1=temp;	
	keywordList=stringToWords(s1);
}

//add a pair of keyword and id of the node which contains this keyword
void CTutor::addMap(string _key,int _nodeID){
	//update keywordToBlock
	JMAP::iterator tempIterator;
	tempIterator = keywordToBlock.find(_key);
	//if _key already has an entry,then just add the BlockID to it
	if(tempIterator!=keywordToBlock.end()){
		(*tempIterator).second.push_back(_nodeID);
	}else{
		VI tempVector;
		tempVector.push_back(_nodeID);
		keywordToBlock.insert(JMAP::value_type(_key,tempVector));
	}
}

string CTutor::readFileNameFromUser(){
	char b[256];
	b[0]='\0';
	string _fileName;	
	while(b[0]=='\0' || b[0]=='\n' ){
		cout<<"Please input file name(example: data.txt):\n";
		cin.getline(b,256,'\n');
	}
	_fileName = b;
	return _fileName;
}

//read text from file
void CTutor::readDataFile(){
	ifstream _file;
	string _fileName = readFileNameFromUser();
	_file.open(_fileName.c_str());
	if(!_file)
		return;
	string buf;
	int  _id =0,_state = _NONE;	
	CBlockNode tempNode;
	string _text; 

	while(getline(_file,buf,'\n')){				
		if(isTitleLine(&_id,buf,&_state,&tempNode))
			continue;
		doReadText(&tempNode,buf,_state,_id);
	}
	blockList.push_back(tempNode);
	_file.close();
}

bool CTutor::isTitleLine(int* pID,string buf,int* pState,CBlockNode* pNode){
	if(buf.find("#key")==0){
		(*pState) = _KEY;
		//reset node;			
		if((*pID) >0)//if not the first line in the file
			blockList.push_back(*pNode);
		pNode->blockID = (*pID);
		(*pID)++;
		return true;
	}
	if(buf.find("#title")==0){
		(*pState) = _TITLE;
		return true;		
	}
	if(buf.find("#text")==0){
		pNode->blockText ="";
		(*pState) = _TEXT;
		return true;		
	}	
	return false;
}
//affiliate function of readDataFile();
//according to _state,invoke different processing part
void CTutor::doReadText(CBlockNode* pTempNode,string buf,int _state,int _id){
	switch(_state){
		case _KEY:{
				//should parse to words
				pTempNode->keywordIncluded = stringToWords(buf);
				for(int i=0;i<pTempNode->keywordIncluded.size();i++){
					string keyString = (pTempNode->keywordIncluded)[i];
					addMap(keyString,(_id-1));
				}
			}
			break;
		case _TITLE:
			pTempNode->blockTitle = buf;
			break;
		case _TEXT:
			pTempNode->blockText=pTempNode->blockText+'\n'+buf;
			break;
	}		
}

//print out the top best match _no blocks
void CTutor::displayResult(int _NO){
	//display the search result, maybe 0 or show 1 add 3 options
	//should judge the range of actually hit number,maybe 0,1,2,3,4,5,etc	
	cout<<"\n Find result: "<<blockHitTimes.size()<<" Blocks\n";
	if(blockHitTimes.size()==0)
		return;
	int _index;
	_index = getMaxValueIndex(0,false);
	cout<<"---------------------------------------------------\n---------------------------------------------------";	
	cout<<"\nThe best match block is :\n";
	printOutBlock(_index,false);
	while(1){
		_NO--;
		if(blockHitTimes.size()<_NO)
			_NO = blockHitTimes.size();
		cout<<"---------------------------------------------------\n---------------------------------------------------";		
		cout<<"\nThe other  "<<_NO<<" match blocks :\n";
		for(int i=0;i<_NO;i++){
			_index = getMaxValueIndex(_index,true);			
			printOutBlock(_index,true);			
		}
		while(1){
			cout<<"---------------------------------------------------\n---------------------------------------------------",	
			cout<<"\nNow if you want to view other blocks releated to this research,please input its number(\"q\" to quit this search):\n";
			string temp;
			cin>>temp;
			if(temp=="q"||temp=="Q")
				return;		
			else{
				int _i = atoi(temp.c_str());
				if(_i==0 && temp!="0")
					return;
				printOutBlock(_i,false);
			}
		}		
	}
}

//test whether this block is alreay in vector
bool CTutor::isNewBlock(int _index){	
	for(int i=0;i<topVector.size();i++){
		if (_index==topVector[i])
			return false;
	}
	return true;
}

//if _less is false, then return the index of node which has Max value 
//if -less if true,  then return the index of node which has Max value, but the Max value 
//                   should no greater than the value indexed by _indexOfMax
int  CTutor::getMaxValueIndex(int _indexOfMax,bool _less){
	int _index=-1;
	int thisMax=0 ,lastMax=0;	
	HITMAP::iterator tempIterator;
	if(_less){
		tempIterator = blockHitTimes.find(_indexOfMax);
		if (tempIterator == blockHitTimes.end())
			return -1;
		lastMax = (*tempIterator).second;
	}
	tempIterator = blockHitTimes.begin();	
	while(tempIterator!=blockHitTimes.end()){
		int i = (*tempIterator).second;
		int j = (*tempIterator).first;
		if(_less){
			if((i>thisMax)&&(i<=lastMax)&&isNewBlock(j) ){//(j!=_indexOfMax)){
				thisMax = i;				
				_index = (*tempIterator).first; 
			}
		}else if(i>thisMax){
				thisMax = i;
				_index = (*tempIterator).first; 
			}
		tempIterator++;			
	}	
	if(_index!=-1)
		topVector.push_back(_index);
	return _index;
}

//record the hit times of every block
void CTutor::addHitTimes(int _nodeID){
	//blockHitTimes	
	int _hitTimes=1;
	HITMAP::iterator tempIterator;
	
	tempIterator = blockHitTimes.find(_nodeID);
	if (tempIterator!=blockHitTimes.end()){
		_hitTimes = (*tempIterator).second;		
		blockHitTimes.erase(tempIterator);
		_hitTimes++;
	}
	blockHitTimes.insert(HITMAP::value_type(_nodeID,_hitTimes));	
}

//do search by keywords(one by one)
void CTutor::doSearch(){
	for(int i=0;i<keywordList.size();i++){
		string temp = keywordList[i];
		JMAP::iterator tempIterator;
		tempIterator = keywordToBlock.find(temp);
		if(tempIterator!=keywordToBlock.end()){
			VI tempVector = (*tempIterator).second;
			for(int j=0;j<tempVector.size();j++){				
				addHitTimes(tempVector[j]);
			}
		}
	}
}

//reset all list, prepare for the next search
void CTutor::resetList(){	
	keywordList.clear();
	keywordToBlock.clear();
	blockList.clear();
	blockHitTimes.clear();
	topVector.clear();
}


//public interface function
void CTutor::doTutor(){
	while(1){			
		resetList();
		readInputKeywords();	
		readDataFile();	
		doSearch();
		printOutHitList();
		displayResult(4);
		cout<<"Do you want to continue another search? (y/n)"<<endl;
		char tempBuf[256];
		tempBuf[0]='\0';
		while(1){
			cin.getline(tempBuf,256,'\n');
			string s=tempBuf;		
			if(s=="n"||s=="N")
				return;
			if(s=="y"||s=="Y")
				break;

		}
	}

}

#ifdef TEST_CTUTOR
void main(int argc, char *argv[]){
	CTutor t;
	t.doTutor();
	return;
}
#endif
